import axios, { AxiosResponse, AxiosRequestConfig, AxiosInstance } from 'axios';
import { host, params, http } from '@/config';
import router from '@/router';
import Utils from '@/extend/Utils';
import { Loading } from 'element-ui';

export default class HttpRequest {
    public static getInstance(...args: any[]): any {
        let Class: any = this;
        if (!Class._instance) {
            let argsLen: number = args.length;
            if (argsLen === 0) {
                Class._instance = new Class();
            } else {
                Class._instance = new Class(...args);
            }
        }
        return Class._instance;
    }

    public loadingInstance: any;
    public queue: any; // 请求的url集合
    public redirect: number;
    public status: number;

    public constructor() {
        this.queue = {};
        this.redirect = 0;
        this.status = 0;
    }

    private getOpts(options: AxiosRequestConfig): AxiosRequestConfig {
        let opts: any = {
            method: 'GET', timeout: 500000, headers: {
                'content-type': 'application/json'
            }, baseURL: host
        };
        opts = Object.assign(opts, options);
        if (opts.method !== 'GET') {
            Object.assign(opts, { data: opts.data });
        } else {
            Object.assign(opts, { params: opts.data });
        }
        return opts;
    }

    public async request(options: AxiosRequestConfig) 
    {
        console.log('url:', options.url)
        let opts = this.getOpts(options);
        const instance = axios.create({
            timeout: 50000,
            baseURL: '/api',
            withCredentials: true,  // send cookies when cross-domain requests
        });
        await this.interceptors(instance, opts);
        
        return instance(opts);
    }

    private interceptors(instance: any, options: AxiosRequestConfig) {
        // 请求拦截
        instance.interceptors.request.use(
            (config: AxiosRequestConfig) => {
                if (options && options.url) {
                    this.queue[options.url] = true;
                }
                return config;
            },
            (error: any) => {
                console.log(error);
            });
        // 响应拦截
        instance.interceptors.response.use(
            (res: AxiosResponse) => {
                if (options && options.url) {
                    this.complete(options.url);
                }
                const { data, status } = res;
                console.log('data =', data)
                console.log('status =', status)
                if (status === 200 && data && data.code === 1) {
                    return data;
                } else {
                    this.redirect = 0;
                    return this.fail(res, options);
                }
            },
            (error: any) => {
                if (options && options.url) {
                    this.complete(options.url);
                }
                console.log(error);
            });
    }



    private async fail(res: AxiosResponse, options: AxiosRequestConfig) {
        if (this.redirect > 3) {
            return false;
        }
        console.log('request fail code:', res);
        switch (res.data.code.toString()) {
            case '10102':
                return res.data;                
            case '20002':
            case '20003':
            case '20004':
            case '20005':
                console.info(res.data.msg);
                this.redirect++;
                delete options.headers;
                return this.request(options);
            case '20001':
            case '20006':
                console.log('需要登录');
                router.replace({ name: 'login' });
                break;
            case '20007':
                console.log('需要完善个人信息');
                router.replace({ name: 'uinfo' });
            default:
                break;
        }

        throw 'Net Error.';
    }

    private complete(url: string) {
        delete this.queue[url];
    }
}
